usrmerge_binsbindeps = {
-- 2ping 389-ds 389-ds-snmp 4ti2 AppStream ...
["/bin/sh"] = 1,
-- 4store AppCSXCAD Cadence ComputeLibrary HepMC-devel ...
["/bin/bash"] = 1,
-- FreeCAD Herwig-libs Mesa Mesa-libEGL1 Mesa-libGL1 ...
["/sbin/ldconfig"] = 1,
-- aaa_base ctdb cvs yast2-printer
["/bin/mktemp"] = 1,
-- avrdude os-prober uisp
["/sbin/modprobe"] = 1,
-- bind
["/sbin/start_daemon"] = 1,
-- budgie-screensaver xscreensaver
["/sbin/unix2_chkpwd"] = 1,
-- canna compiz-branding-SLED compiz-branding-openSUSE compiz-branding-upstream ini4j-javadoc ...
["/bin/rm"] = 1,
-- clamav enscript gpm hplip lsb ...
["/bin/sed"] = 1,
-- clamav icecream lsb
["/bin/tar"] = 1,
-- cobbler
["/sbin/service"] = 1,
-- compiz-branding-SLED compiz-branding-openSUSE compiz-branding-upstream ini4j-javadoc install-initrd-Kubic ...
["/bin/ln"] = 1,
-- corosync eid-mw postgresql10-server
["/sbin/chkconfig"] = 1,
-- dds2tar dt fpc-src lua-lmod petsc-devel ...
["/bin/csh"] = 1,
-- glibc-locale-base jfbterm lsb uisp
["/bin/cat"] = 1,
-- hplip os-prober
["/bin/grep"] = 1,
-- kdevplatform
["/bin/zsh"] = 1,
-- ksh lsb twin
["/bin/true"] = 1,
-- libgtop-doc
["/sbin/install-info"] = 1,
-- logdigest lsb
["/bin/chmod"] = 1,
-- logdigest lsb sound-theme-freedesktop tei_4
["/bin/touch"] = 1,
-- lsb
["/bin/chgrp"] = 1,
["/bin/chown"] = 1,
["/bin/dd"] = 1,
["/bin/df"] = 1,
["/bin/echo"] = 1,
["/bin/false"] = 1,
["/bin/kill"] = 1,
["/bin/mknod"] = 1,
["/bin/more"] = 1,
["/bin/mount"] = 1,
["/bin/mv"] = 1,
["/bin/ps"] = 1,
["/bin/pwd"] = 1,
["/bin/rmdir"] = 1,
["/bin/sleep"] = 1,
["/bin/sort"] = 1,
["/bin/sync"] = 1,
["/bin/umount"] = 1,
["/bin/uname"] = 1,
["/sbin/shutdown"] = 1,
-- lsb lynx
["/bin/cp"] = 1,
-- lsb mariadb openwsman-server patch2mail xorg-x11-Xvnc ...
["/bin/hostname"] = 1,
-- lsb nagios nagios-www ntp sca-appliance-common ...
["/bin/logger"] = 1,
-- lsb perl-Term-ReadKey
["/bin/stty"] = 1,
-- lsb sendmail
["/bin/fuser"] = 1,
-- lsb texlive-cjk-latex-extras
["/bin/mkdir"] = 1,
-- lsb vnstat
["/bin/ls"] = 1,
["/bin/su"] = 1,
-- mdadm
["/sbin/mkinitrd"] = 1,
-- memtest86+
["/sbin/update-bootloader"] = 1,
-- sca-appliance-common
["/bin/ping"] = 1,
-- setserial
["/sbin/isserial"] = 1,
-- sgml-skel xorg-x11-Xvnc
["/bin/awk"] = 1,
-- sysconfig
["/sbin/ifup"] = 1,
["/sbin/netconfig"] = 1,
-- sysconfig-netconfig
["/bin/gawk"] = 1,
-- vpnc
["/sbin/ip"] = 1,
-- xdm
["/sbin/startproc"] = 1,
}
